@@ -1,10 +1,15 @@
 import typing
 from abc import ABC
+from dataclasses import dataclass
 
 import boto3
+from boto3.dynamodb import types
 from botocore.config import Config
 
-from py_aws_core import const, entities, logs, secrets_manager, utils
+from py_aws_core import const, logs, utils
+from .secrets_manager import get_secrets_manager
+
+secrets_manager = get_secrets_manager()
 
 logger = logs.logger
 
@@ -26,7 +31,6 @@ class DDBClient:
     def __init__(self):
         self._boto_client = None
         self._table_resource = None
-        self._secrets_manager = secrets_manager.SecretsManager()
 
     @property
     def boto_client(self):
@@ -53,30 +57,31 @@ class DDBClient:
             verify=False  # Don't validate SSL certs for faster responses
         )
 
+    @classmethod
+    def get_table_name(cls):
+        return secrets_manager.get_secret(secret_name='AWS_DYNAMO_DB_TABLE_NAME')
+
     def get_new_table_resource(self):
         dynamodb = boto3.resource('dynamodb')
         return dynamodb.Table(self.get_table_name())
 
-    def get_table_name(self):
-        return self._secrets_manager.get_secret(secret_name='AWS_DYNAMO_DB_TABLE_NAME')
-
     def query(self, *args, **kwargs):
-        return self.boto_client.query(*args, **kwargs)
+        return self.boto_client.query(TableName=self.get_table_name(), *args, **kwargs)
 
     def scan(self, *args, **kwargs):
-        return self.boto_client.scan(*args, **kwargs)
+        return self.boto_client.scan(TableName=self.get_table_name(), *args, **kwargs)
 
     def get_item(self, *args, **kwargs):
-        return self.boto_client.get_item(*args, **kwargs)
+        return self.boto_client.get_item(TableName=self.get_table_name(), *args, **kwargs)
 
     def put_item(self, *args, **kwargs):
-        return self.boto_client.put_item(*args, **kwargs)
+        return self.boto_client.put_item(TableName=self.get_table_name(), *args, **kwargs)
 
     def delete_item(self, *args, **kwargs):
-        return self.boto_client.delete_item(*args, **kwargs)
+        return self.boto_client.delete_item(TableName=self.get_table_name(), *args, **kwargs)
 
     def update_item(self, *args, **kwargs):
-        return self.boto_client.update_item(*args, **kwargs)
+        return self.boto_client.update_item(TableName=self.get_table_name(), *args, **kwargs)
 
     def batch_write_item(self, *args, **kwargs):
         return self.boto_client.batch_write_item(*args, **kwargs)
@@ -105,20 +110,13 @@ def get_db_client():
     return __db_client
 
 
-class QueryResponse(ABC):
-    def __init__(self, data):
-        self.Items = data.get('Items')
-        self.Count = data.get('Count')
-        self.ScannedCount = data.get('ScannedCount')
-        self.ResponseMetadata = ResponseMetadata(data['ResponseMetadata'])
-
-    def get_by_type(self, _type: str) -> typing.List:
-        if self.Items:
-            return [item for item in self.Items if entities.BaseModel(item).Type == _type]
-        return list()
-
-
 class ABCCommonAPI(ABC):
+    """
+        https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/Expressions.ConditionExpressions.html
+        If your primary key consists of both a partition key(pk) and a sort key(sk),
+        the parameter will check whether attribute_not_exists(pk) AND attribute_not_exists(sk) evaluate to true or
+        false before attempting the write operation.
+    """
     @classmethod
     def get_batch_entity_create_map(
         cls,
@@ -126,7 +124,7 @@ class ABCCommonAPI(ABC):
         sk: str,
         _type: str,
         created_by: str = '',
-        expire_in_seconds: int = const.DB_DEFAULT_EXPIRES_IN_SECONDS,
+        expire_in_seconds: int | None = const.DB_DEFAULT_EXPIRES_IN_SECONDS,
         **kwargs,
     ):
         return {
@@ -140,6 +138,57 @@ class ABCCommonAPI(ABC):
             'ExpiresAt': cls.calc_expire_at_timestamp(expire_in_seconds=expire_in_seconds),
         } | kwargs
 
+    @classmethod
+    def get_entity_update_map(
+        cls,
+        pk: str,
+        sk: str,
+        modified_by: str = '',
+        **kwargs,
+    ):
+        return {
+            'PK': pk,
+            'SK': sk,
+            'ModifiedAt': cls.iso_8601_now_timestamp(),
+            'ModifiedBy': modified_by,
+        } | kwargs
+
+    @classmethod
+    def get_put_item_map(
+        cls,
+        pk: str,
+        sk: str,
+        _type: str,
+        created_by: str = '',
+        expire_in_seconds: int | None = const.DB_DEFAULT_EXPIRES_IN_SECONDS,
+        **kwargs,
+    ):
+        vals = {
+           'PK': pk,
+           'SK': sk,
+           'Type': _type,
+           'CreatedAt': cls.iso_8601_now_timestamp(),
+           'CreatedBy': created_by,
+           'ModifiedAt': '',
+           'ModifiedBy': '',
+           'ExpiresAt': cls.calc_expire_at_timestamp(expire_in_seconds=expire_in_seconds),
+        } | kwargs
+        return cls.serialize_types(vals)
+
+    @staticmethod
+    def serialize_types(data: dict):
+        """
+        Converts normalized json to low level dynamo json
+        """
+        return {k: types.TypeSerializer().serialize(v) for k, v in data.items()}
+
+    @staticmethod
+    def deserialize_types(data: dict):
+        """
+        Converts low level dynamo json to normalized json
+        """
+        return {k: types.TypeDeserializer().deserialize(v) for k, v in data.items()}
+
     @classmethod
     def iso_8601_now_timestamp(cls) -> str:
         return utils.to_iso_8601()
@@ -156,6 +205,18 @@ class ABCCommonAPI(ABC):
             return ''
         return utils.add_seconds_to_current_unix_timestamp(seconds=expire_in_seconds)
 
+    @dataclass
+    class UpdateField:
+        expression_attr: str
+        pk_name: str = 'PK'
+        set_once: bool = False
+
+    @staticmethod
+    def build_update_expression(fields: typing.List[UpdateField]):
+        n_fields = [f'#{f.expression_attr} = :{f.expression_attr}' for f in fields if not f.set_once]
+        o_fields = [f'#{f.expression_attr} = if_not_exists({f.pk_name}, :{f.expression_attr})' for f in fields if f.set_once]
+        return f'SET {', '.join(n_fields + o_fields)}'
+
 
 class ErrorResponse:
     class Error:
@@ -180,7 +241,7 @@ class ErrorResponse:
                 raise exc
 
 
-class DDBItemResponse(ABC):
+class DDBItemResponse:
     def __init__(self, data):
         self._data = data
         self.Type = data.get('__type')
@@ -188,7 +249,31 @@ class DDBItemResponse(ABC):
         self.ResponseMetadata = ResponseMetadata(data.get('ResponseMetadata', dict()))
 
 
-class DynamoDBTransactResponse(ABC):
+class GetItemResponse:
+    def __init__(self, data):
+        self.item = data.get('Item')
+        self.response_metadata = ResponseMetadata(data['ResponseMetadata'])
+
+
+class QueryResponse:
+    def __init__(self, data):
+        self._items = data.get('Items') or list()
+        self.count = data.get('Count')
+        self.scanned_count = data.get('ScannedCount')
+        self.response_metadata = ResponseMetadata(data['ResponseMetadata'])
+
+    def get_by_type(self, _type: str) -> typing.List:
+        if self._items:
+            return [i for i in self._items if i['Type']['S'] == _type]
+        return list()
+
+
+class UpdateItemResponse:
+    def __init__(self, data: typing.Dict):
+        self.attributes = data['Attributes']
+
+
+class DynamoDBTransactResponse:
     def __init__(self, data):
         self._data = data
         self.Responses = data.get('Responses')
