@@ -1,14 +1,12 @@
-import uuid
+import typing
 from abc import ABC, abstractmethod
 
-from boto3.dynamodb import types
-
-from py_aws_core import const, encoders
+from py_aws_core import db_dynamo, encoders
 
 
 class BaseModel:
     def __init__(self, data):
-        self.__data = self.to_normalized_data(data)
+        self.__data = self.deserialize_data(data)
         self.PK = self.data.get('PK')
         self.SK = self.data.get('SK')
         self.Type = self.data.get('Type')
@@ -22,16 +20,13 @@ class BaseModel:
     def data(self):
         return self.__data
 
-    @staticmethod
-    def to_normalized_data(data: dict) -> dict:
-        """
-        Converts low level dynamo json to normalized json
-        """
-        return {k: types.TypeDeserializer().deserialize(v) for k, v in data.items()}
-
     @property
     def to_json(self):
-        return encoders.DBEncoder().serialize_to_json(self)
+        return encoders.JsonEncoder().serialize_to_json(self)
+
+    @staticmethod
+    def deserialize_data(data: dict) -> typing.Dict:
+        return db_dynamo.ABCCommonAPI.deserialize_types(data)
 
 
 class ABCEntity(ABC, BaseModel):
@@ -52,8 +47,15 @@ class Session(ABCEntity):
 
     def __init__(self, data):
         super().__init__(data)
-        self.Base64Cookies = self.data['Base64Cookies']
+        self.Base64Cookies = self.data.get('Base64Cookies')
+        self.SessionId = self.data.get('SessionId')
 
     @classmethod
-    def create_key(cls, _id: uuid.UUID) -> str:
+    def create_key(cls, _id: str) -> str:
         return f'{cls.type()}#{str(_id)}'
+
+    @property
+    def b64_cookies_bytes(self):
+        if self.Base64Cookies:
+            return self.Base64Cookies.value
+        return None
