@@ -10,6 +10,7 @@ from datetime import datetime, timezone, timedelta, UTC
 def build_lambda_response(
     status_code: int,
     body: typing.Dict | str = None,
+    content_type: str = 'application/json',
     multi_value_headers: typing.Dict[str, typing.List[str]] = None,
     exc: Exception = None
 ):
@@ -19,9 +20,11 @@ def build_lambda_response(
       See below for further details:
       https://stackoverflow.com/a/54089431
       https://docs.aws.amazon.com/apigateway/latest/developerguide/how-to-cors.html#apigateway-enable-cors-proxy
-    :param multi_value_headers:
+
     :param status_code:
     :param body:
+    :param content_type:
+    :param multi_value_headers:
     :param exc:
     :return:
     """
@@ -32,7 +35,7 @@ def build_lambda_response(
     if isinstance(body, dict):
         body = json.dumps(body)
     response_headers = {
-        'Content-Type': ['application/json'],
+        'Content-Type': [content_type],
         'Access-Control-Allow-Credentials': [True],
         'Access-Control-Allow-Headers': ['Content-Type,X-Amz-Date,Authorization,X-Api-Key,X-Amz-Security-Token'],
         'Access-Control-Allow-Origin': ['*'],
@@ -85,8 +88,8 @@ def sleep(seconds: float) -> None:
     return time.sleep(seconds)
 
 
-def get_uuid() -> uuid.UUID:
-    return uuid.uuid4()
+def get_uuid_hex() -> str:
+    return uuid.uuid4().hex
 
 
 def get_environment_variable(secret_name: str, default=None) -> str:
@@ -111,4 +114,8 @@ def unix_timestamp_to_iso8601(unix_ts: int):
 
 
 def remove_whitespace(s: str) -> str:
-    return ''.join([line.replace(' ', '') for line in s.split('\\r\\n')])
+    return ''.join([line.strip() for line in s.split('\\r\\n')])
+
+
+def to_utf8_bytes(s: str) -> bytes:
+    return bytes(s, 'utf-8')
